package gecco.game.dke;

import java.util.Vector;
import gecco.server.automaton.AutomatonMap;
import gecco.server.core.*;
import java.util.Enumeration;
import gecco.server.visibility.Point;


/**
 * RouteFinder.java
 *
 *
 * Created: Sun Oct 20 20:41:22 2002
 *
 * @author <a href="mailto:Hans.Sandstrom@fhs.mil.se"> Hans Sandstrm </a>
 * @version
 */

public class RouteFinder {

    static int lastRouteLinkIndex = 0;

    static public Point[] findRoute(AutomatonImpl start, Point startMapIndex,
			     AutomatonImpl end, Point endMapIndex){
	return findRoute(start, startMapIndex, end, endMapIndex, false, 0);
    }

    static public boolean isRouteOpen(Point startMapIndex, Point endMapIndex, int force){
	AutomatonImpl start = (AutomatonImpl) ReferenceHolder.getAutomatonMap().getAutomaton(startMapIndex.x, 
											     startMapIndex.y);
	AutomatonImpl end = (AutomatonImpl) ReferenceHolder.getAutomatonMap().getAutomaton(endMapIndex.x,
											   endMapIndex.y);
	Point[] openRoute = findRoute(start, startMapIndex, end, endMapIndex, true, force);

	if (openRoute == null) {
	    return false;
	}
	else {
	    return true;
	}
    }

    /**
     * Finds a route from start to end. Start excluded, end included.
     * Bredden frst ger den optimala vgen (i hela Automatons)
     *
     * @param start an <code>AutomatonImpl</code> value
     * @param end an <code>AutomatonImpl</code> value
     * @return an <code>AutomatonImpl[]</code> value containing the route or null if none are found.
     */
    static synchronized public Point[] findRoute(AutomatonImpl start, Point startMapIndex,
					  AutomatonImpl end, Point endMapIndex,
					  boolean testForLastVisitor, int force){
	
	// Check that start and end is a route
	if (!start.isRoute()) {
	    return null;
	}
	if (!end.isRoute()) {
	    return null;
	}
	
	Vector routeLinkVector = new Vector();
	AutomatonMap map = ReferenceHolder.getAutomatonMap();
	boolean lastVisitorOK = testForLastVisitor? false: true;

	// Add start
	routeLinkVector.add(new RouteLink(start, RouteFinder.lastRouteLinkIndex, -1, startMapIndex));
	start.setVisited(true);


	int i = 0;
	while (i <= RouteFinder.lastRouteLinkIndex) {
	    RouteLink rl = (RouteLink)routeLinkVector.get(i);
	    
	    if (rl.getCurrentAutomaton()== end) {
		break; // We have found a path
	    }
	    
	    Point p = rl.getMapIndex();
	    for (int  y = -1; y < 2; y++) {
		for (int x = -1; x < 2; x++) {
		    if (!(x == 0 && y == 0)){
			if (map.areValidCoordinates(p.x + x, p.y + y)) {
			    AutomatonImpl ai = (AutomatonImpl)map.getAutomaton(p.x + x, p.y + y);
			    if (testForLastVisitor) {
				lastVisitorOK = (force == ai.getLastVisitor())? true: false;
			    }
			    if (ai.isRoute() && !ai.isVisited() && lastVisitorOK) {
				ai.setVisited(true);
				routeLinkVector.add(new RouteLink(ai, ++RouteFinder.lastRouteLinkIndex,
								  rl.getRouteLinkIndex(),
								  new Point(p.x + x, p.y + y)));
			    }
			}
		    }
		}
	    }

	    i++;
	} 

	if (i > RouteFinder.lastRouteLinkIndex) { // There were no route between start and end
	    // Reset visited flag and RoueLink count
	    Enumeration e = routeLinkVector.elements();
	    while (e.hasMoreElements()) {
		((RouteLink)e.nextElement()).getCurrentAutomaton().setVisited(false);
	    }
	    RouteFinder.resetCounter();
	    return null;
	}
	
	Vector v = new Vector();

	// Get the route in correct order
	while (i > 0) {
	    v.add(0, ((RouteLink)routeLinkVector.get(i)).getMapIndex());
	    i = ((RouteLink)routeLinkVector.get(i)).getUp();
	}

	// Reset visited flag and RoueLink count
	Enumeration e = routeLinkVector.elements();
	while (e.hasMoreElements()) {
	    ((RouteLink)e.nextElement()).getCurrentAutomaton().setVisited(false);
	}
	RouteFinder.resetCounter();

	Object[] oArray = v.toArray();
	Point[] pArray = new Point[oArray.length];
	for (int j = 0; j < oArray.length; j++) {
	    pArray[j] = (Point)oArray[j];
	}

	return pArray;
    }

    static void resetCounter(){
	lastRouteLinkIndex = 0;
    }

}//RouteFinder
