package gecco.game.dke3;

import gecco.game.dke.RefereeTimerThread;
import gecco.game.dke.Def;
import gecco.server.core.ReferenceHolder;
import java.util.*;
import gecco.server.visibility.Point;
import gecco.game.dke.RouteFinder;
import gecco.server.core.Unit;



/**
 * DKE3Referee.java
 *
 *
 * Created: Mon Jan 19 16:31:31 2004
 *
 * @author <a href=mailto:hans.sandstrom@fhs.mil.se> Hans Sandstrm </a>
 * @version 1.0
 */

public class DKE3Referee extends gecco.game.dke.DKEReferee {

    public DKE3Referee() {
	super();
    }
    
    public void startReferee(){
	new RefereeTimerThread(this, DefDKE3.getMaxGameTime(), DefDKE3.getMaxGameTime());
	setInitialOpenRoute();
    }

    private void setInitialOpenRoute(){
	int range = DefDKE3.getInitialOpenRouteRange();
	Set set;
	AutomatonImplDKE3 automaton;
	Iterator it;
	// start with blue force
	set = new HashSet();
	it = getWarUnits().iterator();
	while (it.hasNext()) {
	    WarUnitDKE3 unit = (WarUnitDKE3) it.next();
	    if (unit.getForce() == DefDKE3.BLUE_FORCE) {
		set.addAll(unit.getAutomatonsWithin(range));
	    }
	}
	it = set.iterator();
	while (it.hasNext()) {
	    automaton = (AutomatonImplDKE3) it.next();
	    if (automaton.isRoute()) {
		automaton.setLastVisitor(DefDKE3.BLUE_FORCE, true);
	    }
	}
	// and now red force
	set.clear();
	it = getWarUnits().iterator();
	while (it.hasNext()) {
	    WarUnitDKE3 unit = (WarUnitDKE3) it.next();
	    if (unit.getForce() == DefDKE3.RED_FORCE) {
		set.addAll(unit.getAutomatonsWithin(range));
	    }
	}
	it = set.iterator();
	while (it.hasNext()) {
	    automaton = (AutomatonImplDKE3) it.next();
	    if (automaton.isRoute()) {
		automaton.setLastVisitor(DefDKE3.RED_FORCE, true);
	    }
	}
	
	
	
    }

    public void timeIsUp(double time){
	if (time == DefDKE3.getMaxGameTime()) {
	    gameOver(); 
	}
    }

    /**
     * <code>lastVisitorChanged</code> overrided 
     * for compatibility reasons
     *
     * @param newVisitor an <code>int</code> value
     */
    public void lastVisitorChanged(int newVisitor){
	return;
    }

    private ArrayList getWarUnits(){
	Collection allUnits = ReferenceHolder.getUnitManager().getAllUnits();
	return new ArrayList(filterOutWarUnits(allUnits));
    }

    private ArrayList filterOutBeachHeadPoints(Collection c){
	ArrayList array = new ArrayList();
	Iterator it = c.iterator();
	while (it.hasNext()) {
	    Object o = it.next();
	    if (o instanceof BeachHeadPoint) {
		array.add(o);
	    }
	}
	return array;
    }

    private ArrayList filterOutBasePoints(Collection c){
	ArrayList array = new ArrayList();
	Iterator it = c.iterator();
	while (it.hasNext()) {
	    Object o = it.next();
	    if (o instanceof BasePoint) {
		array.add(o);
	    }
	}
	return array;
    }

    private ArrayList getBeachHeadPoints(){
	Collection allUnits = ReferenceHolder.getUnitManager().getAllUnits();
	 return filterOutBeachHeadPoints(allUnits);
    }

    protected  ArrayList getBasePoints(){
	Collection allUnits = ReferenceHolder.getUnitManager().getAllUnits();
	 return filterOutBasePoints(allUnits);
    }

    private BasePoint getBasePointFor(int force){
	Collection basePoints = getBasePoints();
	Iterator it = basePoints.iterator();
	while (it.hasNext()) {
	    BasePoint bp = (BasePoint)it.next();
	    if (bp.getBaseForForce() == force){
		return bp;
	    }
	}
	return null;
    }

    private boolean isBeachHeadCaptured(ArrayList aBeachHead){
	if (aBeachHead.isEmpty()) {
	    // Debug info
	    ReferenceHolder.out.println("No points found in beachHead");
	    return false;
	}
	int nrOfBeachHeadPoints = aBeachHead.size();
	BeachHeadPoint bhp = (BeachHeadPoint) aBeachHead.get(0);
	int owner = bhp.getOwner();
	if (owner == Def.NO_FORCE) {
	    // Debug info
	    ReferenceHolder.out.println("No owner of beachHead " + bhp.getName());
	    return false;
	}
	// only one beachHeadPoint
	if (nrOfBeachHeadPoints < 2) {
	    // Debug info
	    ReferenceHolder.out.println("Single beachHead " + bhp.getName() + " captured");
	    return true; // all tests ok
	}
	else {
	    // Kontroll av samma owner
	    for (int i = 1; i < nrOfBeachHeadPoints; i++) {
		bhp = (BeachHeadPoint) aBeachHead.get(i);
		if (owner != bhp.getOwner()) {
		    // Debug info
		    ReferenceHolder.out.println("Not the same owner of beachHead " + bhp.getName());
		    return false;
		}
	    }
	    // Kontroll av ppen vg och maxavstnd, grs i isBeachHeadRouteOpen()
	    bhp = (BeachHeadPoint) aBeachHead.get(0);
	    Point startPoint = new Point(bhp.getIntX(), bhp.getIntY());
	    for (int i = 1; i < nrOfBeachHeadPoints; i++) {
		if (!RouteFinderDKE3.isBeachHeadRouteOpen(bhp, (BeachHeadPoint) aBeachHead.get(i))) {
		     return false;
		}
	    }	    
	}
	// all tests passed
	// Debug info
	ReferenceHolder.out.println("beachHead " + bhp.getName() + " captured");
	return true;
    }

    /**
     * <code>getCapturedBeachHeads</code> returns a arrayList containing ArrayLists with
     * the BeachHeadPoints that are part of a captured beach-head.
     *
     * @return an <code>ArrayList</code> value
     */
    protected ArrayList getCapturedBeachHeads(){
	ArrayList allBeachHeadPoints = getBeachHeadPoints();
	ArrayList capturedBeachHeads = new ArrayList();
	while (!allBeachHeadPoints.isEmpty()) {
	    BeachHeadPoint beachHeadPoint = (BeachHeadPoint) allBeachHeadPoints.get(0);
	    ArrayList aBeachHead = new ArrayList(4);
	    aBeachHead.add(beachHeadPoint);
	    allBeachHeadPoints.remove(beachHeadPoint);
	    String beachHeadName = beachHeadPoint.getName();
	    Iterator it = allBeachHeadPoints.iterator();
	    // Sort out a beachHead
	    while (it.hasNext()) {
		beachHeadPoint = (BeachHeadPoint) it.next();
		if (beachHeadPoint.getName().equals(beachHeadName)) {
		    aBeachHead.add(beachHeadPoint);
		    it.remove();
		}
	    }
	    if (isBeachHeadCaptured(aBeachHead)) {
		capturedBeachHeads.add(aBeachHead);
	    }
	}
	return capturedBeachHeads;
    }

    private  void  decideWinner(){
	winningForce = Def.NO_FORCE;
	ArrayList capturedBeachHeads = getCapturedBeachHeads();
	if (capturedBeachHeads.isEmpty()) {
	    return;
	}
	ArrayList basePoints = getBasePoints();
	Iterator it = capturedBeachHeads.iterator();
	while (it.hasNext()) {
	    ArrayList beachHead = (ArrayList) it.next();
	    BeachHeadPoint bhp = (BeachHeadPoint) beachHead.get(0);
	    if (winningForce == Def.NO_FORCE) {
		BasePoint base = getBasePointFor(bhp.getOwner());
		if (RouteFinderDKE3.isGameEventPointRouteOpen(bhp, base)) {
		    winningForce = bhp.getOwner();
		}
	    }
	    else {
		if (winningForce != bhp.getOwner()) {
		    BasePoint base = getBasePointFor(bhp.getOwner());
		    if (RouteFinderDKE3.isGameEventPointRouteOpen(bhp, base)) {
			winningForce = Def.NO_FORCE;
			return;
		    }
		}
	    }
	} 
	return;
    }

    protected void gameOver(){
	stopGame();
	calculateScore();
	decideWinner();
	writeResult();
	String message = Def.getWinnerIsString() + Def.getForceName(winningForce);
	ReferenceHolder.getClient().gameOver(message);
	ReferenceHolder.out.println("Game stoped by referee, game over.");
    }

    public void writeResult(){
	writeResult(ReferenceHolder.out);
	writeSpecifiedResult(ReferenceHolder.out);
	if (ReferenceHolder.log != null) {
	    writeResult(ReferenceHolder.log);
	    writeSpecifiedResult(ReferenceHolder.log);
	    writeUnitsSpecification(ReferenceHolder.log);
	}
    }
	
    private void writeResult(java.io.PrintWriter out){
	out.println("************ Game Result *************");
	out.println("Winner   : " + Def.getForceName(winningForce));
	out.println("Game ended at " + ReferenceHolder.getTimeString().getGameTime() + " game time.");
	out.println("Game type: " + gameType);
	out.println("******** End of Game Result **********");
	out.println();
    }

    private void writeSpecifiedResult(java.io.PrintWriter out){
	out.println("************ Specified Result *************");
	out.println(Def.getForceName(Def.BLUE_FORCE) + " remaining units       : " + blueRemainingUnits);
	out.println(Def.getForceName(Def.BLUE_FORCE) + " score from WarUnits   : " + blueWarUnitsPoints);
	out.println(Def.getForceName(Def.RED_FORCE) + " remainging units      : " + redRemainingUnits);
	out.println(Def.getForceName(Def.RED_FORCE) + " score from WarUnits   : " + redWarUnitsPoints);
	out.println("******** End of Specified Result **********");
	out.println();
    }

    private void writeUnitsSpecification(java.io.PrintWriter out){
	out.println("************ War Units *************");
	Collection c = filterOutWarUnits(ReferenceHolder.getUnitManager().getAllUnits());
	Iterator it = c.iterator();
	while (it.hasNext()) {
	    Unit u = (Unit) it.next();
	    u.printWriteMe(out);
	    out.println();
	}
	out.println("************ End of War Units *************");
	out.println();
	out.println("************ BeachHead Points *************");
	c = filterOutBeachHeadPoints(ReferenceHolder.getUnitManager().getAllUnits());
	it = c.iterator();
	while (it.hasNext()) {
	    Unit u = (Unit) it.next();
	    u.printWriteMe(out);
	    out.println();
	}
	out.println("************ End of BeachHead Points *************");
	out.println();
    }

} // DKE3Referee
