package gecco.game.dke3;

import gecco.game.dke.*;
import gecco.server.core.Unit;
import gecco.server.core.ReferenceHolder;
import java.util.*;

/**
 * WarUnitDKE3.java
 *
 *
 * Created: Mon Jan 19 12:08:24 2004
 *
 * @author <a href=mailto:hans.sandstrom@fhs.mil.se> Hans Sandstrm </a>
 * @version 1.0
 */
public class WarUnitDKE3 extends gecco.game.dke.WarUnit {

    private  boolean autoFire;

    public WarUnitDKE3() {
	super();
    }

    public void init(){
	super.init();
	// om tillgnglig, lgg AutoFireActionOffAction bland de blockerade
	if (getUnitType().getActionNames().contains(DefDKE3.getAutoFireOffActionName())) {
	    addBlockedAction(DefDKE3.getAutoFireOffActionName());
	}
    }
    
    /**
     * Get the AutoFire value.
     * @return the AutoFire value.
     */
    public boolean isAutoFire() {
	return autoFire;
    }

    /**
     * Set the AutoFire value.
     * @param newAutoFire The new AutoFire value.
     */
    public void setAutoFire(boolean newAutoFire) {
	this.autoFire = newAutoFire;
    }

    /**
     * <code>isReadyForAutoIndirectFire</code>
     *
     * @return a <code>boolean</code> value
     */
    public boolean isReadyForAutoIndirectFire(){
	// do tests
	if (getIndirectFireFactor() < 1) return false;
 	if (isMarching()) return false;
 	if (isAdvancing()) return false;
	if (isReloading()) return false;
	if (isFiringIndirect()) return false;
	if (isRegainingStrength()) return false;

	return true;
    }

    /**
     * <code>getTargetForAutoIndirectFire</code> returns a WarUnit from
     * a alian force that is in combat and is whithin fire range. If no such WarUnit
     * is found <code>null</code> is returned
     *
     * @return an <code>Unit</code> value
     */
    public Unit getTargetForAutoIndirectFire(){
	Iterator it = ReferenceHolder.getUnitManager().getAllUnits().iterator();
	TreeMap possibleTargets = new TreeMap();

	// Sortera ut de ml (med avseende p avstnd) som r tnkbara fr autoIndirectFire
	while (it.hasNext()) {
	    Unit u = (Unit) it.next();
	    if (u instanceof WarUnit) {
		double distance = getDistanceToUnit(u);
		if (Def.isWhithinIndirectFireRange(distance)) {
		    WarUnit wu = (WarUnit) u;
		    if (getForce() != wu.getForce()) {
			possibleTargets.put(new Double(distance), wu);
		    }
		}
	    }
	}
	// Vlj ut ml
	it = possibleTargets.values().iterator();
	while (it.hasNext()) {
	    WarUnit wu = (WarUnit) it.next();
	    if (wu.isInCombat()) {
		if (!wu.isDestroyed()){
		    return wu;
		}
 	    }
	}
	return null;
    }

    /**
     * <code>getDistanceToUnit</code> returns the distance in map pixels to
     * Unit u
     *
     * @param u an <code>Unit</code> value
     * @return a <code>double</code> value
     */
    private double getDistanceToUnit(Unit u){
	double x = getX();
	double y = getY();
	double dx = x - u.getX();
	double dy = y - u.getY();
	double distance = Math.sqrt(dx*dx + dy*dy);
	return distance;
    }

    /**
     * <code>getIndirectFireExtendTime</code> overrides parent
     *
     * @return a <code>double</code> value
     */
    public double getIndirectFireExtendTime(){
	return DefDKE3.getIndirectFireExtendTime(this);
    }

    /**
     * <code>getReloadExtendTime</code> overrides parent
     *
     * @return a <code>double</code> value
     */
    public double getReloadExtendTime(){
	return DefDKE3.getReloadExtendTime();
    }

    /**
     * <code>getTimeToShowIndirectFire</code overrides parent
     *
     * @return a <code>double</code> value
     */
    public double getTimeToShowIndirectFire(){
	return DefDKE3.getTimeToShowIndirectFire();
    }
	
    /**
     * <code>getDisturbedTime</code> overrides parent
     
     * @return a <code>double</code> value
     */
    public double getDisturbedTime(){
	return DefDKE3.getDisturbedTime();
    }

      /**
     * <code>getCombatExtendTime</code> overrides parent
     *
     * @return a <code>double</code> value
     */
    public double getCombatExtendTime(){
	return DefDKE3.getCombatExtendTime();
    }

    /**
     * <code>getRegainStrengthTime</code> overrides parent
     *
     * @return an <code>int</code> value
     */
    public int getRegainStrengthTime(){
	return DefDKE3.getRegainStrengthTime();
    }

    /**
     * <code>getAutomatonsWithin</code> returns the Automatons 
     * that is within range from the unit
     *
     * @param range an <code>int</code> value
     * @return a <code>Collection</code> value
     */
    public Collection getAutomatonsWithin(int range){
	ArrayList list = new ArrayList(range*range);
	int unitX = getIntX();
	int unitY = getIntY();
	int dx;
	int dy;
	double distance;
	gecco.server.automaton.AutomatonMap am = ReferenceHolder.getAutomatonMap();
	for (int y  = unitY - range; y <= unitY + range; y++) {
	    for (int x  = unitX - range; x <= unitX + range; x++) {
		if (am.areValidCoordinates(x,y)) {
		    dx = x - unitX;
		    dy = y - unitY;
		    distance = Math.sqrt(dx*dx + dy*dy);
		    if (distance <= range) {
			list.add(am.getAutomaton(x,y));
		    }
		}
	    }
	}
	return list;
    }
	
    /**
     * Overrides parent
     *
     * Returns the time that shall pass before next update of movement. The method 
     * uses <code>Def.STEP_LENGTH, Def.MAP_SCALE</code>, the property "Rrlighet"
     * and <code>groundState</code>
     *
     * @return seconds to next update
     */
    double getTimeToRequeueForMovement() {
	return getTimeToRequeueForMovement(DefDKE3.STEP_LENGTH);
    }
    
    /**
     * Overrides parent
     *
     * Returns the time that shall pass before next update of movement. The method 
     * uses <code>Def.STEP_LENGTH, Def.MAP_SCALE</code>, the property "Rrlighet"
     * and <code>groundState</code>
     *
     * @return seconds to next update
     */
    double getTimeToRequeueForMovement(double stepLength) {
	String speedLevel = getStringProperty(DefDKE3.getMovementPropertyString());
	double disturbedFactor = isDisturbed()? DefDKE3.getDisturbedPerformanceFactor(): 1;
	double terrainDenominator = DefDKE3.getTerrainSpeedDenominator(getGroundState());
	double speedFactor = DefDKE3.speedFactorConverter(getSpeedFactor());
	double fuelFactor = getFuelLevelSpeedFactor();
	double currentSpeed = DefDKE3.getSpeed(speedLevel)*disturbedFactor/terrainDenominator*speedFactor*fuelFactor;
	double secondsToUpdate = stepLength * DefDKE3.MAP_SCALE / currentSpeed;
	return secondsToUpdate;
    }

    /**
     * Overrides parent
     *
     * In DKE3 we don't use combat result messages
     *
     * @param ownDamage an <code>int</code> value
     * @param enemyDamage an <code>int</code> value
     */
    public void sendCombatEndedMessage(int ownDamage, int enemyDamage){
	return;
    }

    /**
     * Overrides parent
     *
     * Only the format differs (added linefeed)
     *
     */

    private final int UPDATE_STATUS = 10;
    public void updateStatus(){
	if (!isUnderAttack() && !isUnderIndirectFire() && !isAttacking() && !isFiringIndirect() &&
	    !isDisturbed() && !isMarching() && !isAdvancing() && !isReloading() && !isRegainingStrength()) {
	    setProperty(Def.getStatusPropertyString(), " ");
	}
	else {
	    String statusString = (isDisturbed() ? "Disturbed\n": "") + (isAttacking() ? "Attacking\n": "") +
		(isReloading() ? "Reloading\n": "") + (isRegainingStrength() ? "Recovering\n": "") +
		(isUnderAttack() ? "Under attack\n": "") + (isFiringIndirect() ? "Firing indirect\n": "") +
		(isUnderIndirectFire() ? "Under fire\n": "") + (isMarching() ? "Moving\n": "") +
		(isAdvancing() ? "Advancing\n": "");
	    setProperty(Def.getStatusPropertyString(), statusString);
	}
	// Don't record replay data when replaying
	if (!ReferenceHolder.inReplayMode()) {
	    Object[] parameters = {null};
	    saveReplayData(UPDATE_STATUS, parameters);
	}
    }

 } // WarUnitDKE3
